/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus,
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "config.hh"

#include "genotype_score.hh"
#include "tree.hh"
#include "tree_builder.hh"

#include <snpfile/array_matrix.hh>
using namespace BiRC::SNPFile;

#undef NDEBUG
#include <cassert>

int main()
{
    Matrix m(newArrayMatrix(10,3,true));
    m(0,0) = 1; 
    m(1,0) = 1; 
    m(2,0) = 1;
    m(3,0) = 1;
    m(4,0) = 1;
    m(0,1) = 1; 
    m(1,1) = 1; 
    m(2,1) = 1;
    m(4,1) = 1;
    m(0,2) = 1; 

    std::vector<int> positions;
    positions.push_back(1);
    positions.push_back(2);
    positions.push_back(3);
   
    //int classes[] = { 0, 0, 0, 0, 0, 0, 1, 1, 1, 1};
    int classes[] = { 0, 0, 0, 1, 1};
    Classification classification(classes, classes+m.noRows()/2);


    for(int i = 1; i <= 3; i++)
    {
        std::cout << std::endl << "With " << i << " mutation(s)." << std::endl;
        GenotypeScore gs(m, positions, 0, -1, classification, i);

        MatrixView w = m.view();
        std::auto_ptr<Tree> t(buildTree(w, 0, 0, positions));
        std::cout << *t << std::endl;

        long double score = gs.scoreLocus(0);
        
        std::cout << std::endl << "Final score was: " << score << std::endl;

        std::cout << "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n";
    }

    return 0;
}
