/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus,
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "config.hh"
#include "incompatibility.hh"
#include "case_control_score.hh"

#include <snpfile/array_matrix.hh>
using namespace BiRC::SNPFile;

#include <iostream>

#undef NDEBUG
#include <cassert>

int
main()
{
    Matrix m(newArrayMatrix(5,8));
    m(0,1) = m(0,2) = 1; 
    m(1,0) = 1; 
    m(2,4) = m(2,5) = m(2,7) = 1;
    m(3,3) = m(3,4) = m(3,7) = 1;
    m(4,3) = m(4,5) = 1;

    std::vector<int> positions;
    positions.push_back(10);
    positions.push_back(13);
    positions.push_back(20);
    positions.push_back(30);
    positions.push_back(40);
    positions.push_back(50);
    positions.push_back(78);
    positions.push_back(80);
    std::vector<int> leftBreaks, rightBreaks;

    int expectedLeftBreaks[]  = { 0, 0, 0, 0, 4, 5, 6, 6 };
    int expectedRightBreaks[] = { 4, 4, 4, 4, 5, 7, 8, 8 };

    int classes[] = { 1, 1, 1, 1, 1};
    Classification classification(classes, classes+m.noRows());

    for (int i = 0; i < m.noCols(); ++i)
	{
	    int leftBreak, rightBreak;
	    calculateIncompatibilities(m, positions,
				       i, leftBreak, rightBreak,
				       0, 0.0);
	    if (leftBreak  != expectedLeftBreaks[i])  return 2;
	    if (rightBreak != expectedRightBreaks[i]) return 2;
	}
}
