/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus,
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "config.hh"
#include "io.hh"

#include <snpfile/matrix.hh>
#include <snpfile/array_matrix.hh>
using namespace BiRC::SNPFile;

#include <iostream>
#include <sstream>

#undef NDEBUG
#include <cassert>

int main()
{
    std::string unorderdpositions = "2 5 3 8 4 9";
    try {
	std::istringstream is(unorderdpositions);
	readPositions(is);
	assert(!"shouldn't reach this point");
    } catch (UnsortedPositions&) {}

    std::string orderdpositions   = "20 30 40 50 80 98 ";
    std::istringstream is(orderdpositions);
    std::vector<int> positions = readPositions(is);
    std::vector<int>::const_iterator i;
    std::ostringstream os;
    for (i = positions.begin(); i != positions.end(); ++i)
	os << *i << ' ';
    assert(os.str() == orderdpositions);
	

    std::string matrix =
	"1 0 0 0 0 0 0 \n"
	"1 1 0 0 0 0 1 \n"
	"0 1 1 0 1 1 0 \n"
	"0 0 0 1 1 0 1 \n";
    std::istringstream matrix_is(matrix);
    
    Matrix m(newArrayMatrix(0,0));
    std::vector<int> c;
    readHaplotypes(matrix_is, m, c);

    std::ostringstream matrix_os; 
    std::string strippedMatrix =
	"0 0 0 0 0 0 \n"
	"1 0 0 0 0 1 \n"
	"1 1 0 1 1 0 \n"
	"0 0 1 1 0 1 \n";
    matrix_os << m;
    assert(strippedMatrix == matrix_os.str());

    assert(c[0] == 1);
    assert(c[1] == 0);


    std::string sameMatrix =
	"1 0 0 0 0 0   0 \n"
	"1 1 0   0 0 0 1 \n"
	"0 1 1 0  1 1 0 \n"
	"0   0 0 1 1 0 1 \n";
    std::istringstream sameMatrix_is(sameMatrix);
    
    readHaplotypes(sameMatrix_is, m, c);

    matrix_os.str("");
    matrix_os << m;
    assert(strippedMatrix == matrix_os.str());

    assert(c[0] == 1);
    assert(c[1] == 0);

    try {
	std::string errorMatrix =
	    "1 0 0 0 0   0 \n"
	    "1 1 0   0 0 0 1 \n"
	    "0 1 1 0  1 1 0 \n"
	    "0   0 0 1 1 0 1 \n";
	std::istringstream error_is(errorMatrix);
	readHaplotypes(error_is, m, c);
	assert(!"shouldn't reach this point");
    } catch(ParseError&) {}

    try {
	std::string errorMatrix = "";
	std::istringstream error_is(errorMatrix);
	readHaplotypes(error_is, m, c);
	assert(!"shouldn't reach this point");
    } catch(ParseError&) {}
}
