/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus,
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "config.hh"
#include "matrix_utils.hh"

#include <snpfile/array_matrix.hh>
using namespace BiRC::SNPFile;

#include <iostream>
#include <sstream>

#undef NDEBUG
#include <cassert>


void fillMatrix(Matrix &m, int val)
{
    for(int col = 0; col < m.noCols(); col++)
        for(int row = 0; row < m.noRows(); row++)
            m(row, col) = val;
}


int
main()
{
    std::ostringstream strstrm;
    std::string expected;

    Matrix m(newArrayMatrix(5,5));
    fillMatrix(m, 0);

    expected = 
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n";
    strstrm << m;
    assert(expected == strstrm.str());
    strstrm.str("");

    m(0,1) = 1; 
    m(0,2) = 1; 
    m(1,0) = 1; 
    m(2,4) = 1;
    m(4,3) = 1;

    expected =
	"0 1 1 0 0 \n"
	"1 0 0 0 0 \n"
	"0 0 0 0 1 \n"
	"0 0 0 0 0 \n"
	"0 0 0 1 0 \n";
    strstrm << m;
    assert(expected == strstrm.str());
    strstrm.str("");


    MatrixView v = m.subMatrix(0,5,0,1);

    expected =
	"0 \n"
	"1 \n"
	"0 \n"
	"0 \n"
	"0 \n";
    strstrm << v;
    assert(expected == strstrm.str());
    strstrm.str("");



    MatrixView w = m.subMatrix(0,1,0,5);

    expected =
	"0 1 1 0 0 \n";
    strstrm << w;
    assert(expected == strstrm.str());
    strstrm.str("");

    expected =
	"0 1 1 \n"
	"1 0 0 \n";
    strstrm << m.subMatrix(0,2,0,3);
    assert(expected == strstrm.str());
    strstrm.str("");

    expected =
	"0 0 0 \n"
	"0 1 0 \n";
    strstrm << m.subMatrix(3,5,2,5);
    assert(expected == strstrm.str());
    strstrm.str("");

    expected =
	"0 \n"
	"0 \n";
    strstrm << m.subMatrix(3,5,2,5).subMatrix(0,2,2,3);
    assert(expected == strstrm.str());
    strstrm.str("");


    v = m.subMatrix(0,m.noRows(), 0,m.noCols());
    expected =
	"0 1 1 0 0 \n"
	"1 0 0 0 0 \n"
	"0 0 0 0 1 \n"
	"0 0 0 0 0 \n"
	"0 0 0 1 0 \n";
    strstrm << v;
    assert(expected == strstrm.str());
    strstrm.str("");

    v = m.subMatrix(0,3,          0,m.noCols());

    expected =
	"0 1 1 0 0 \n"
	"1 0 0 0 0 \n"
	"0 0 0 0 1 \n";
    strstrm << v;
    assert(expected == strstrm.str());
    strstrm.str("");

    w = m.subMatrix(3,m.noRows(), 0,m.noCols());
    expected =
	"0 0 0 0 0 \n"
	"0 0 0 1 0 \n";
    strstrm << w;
    assert(expected == strstrm.str());
    strstrm.str("");




    v = m.subMatrix(0,m.noRows(), 0,2);
    expected =
	"0 1 \n"
	"1 0 \n"
	"0 0 \n"
	"0 0 \n"
	"0 0 \n";
    strstrm << v;
    assert(expected == strstrm.str());
    strstrm.str("");

    

    w = m.subMatrix(0,m.noRows(), 2,m.noCols());
    expected =
	"1 0 0 \n"
	"0 0 0 \n"
	"0 0 1 \n"
	"0 0 0 \n"
	"0 1 0 \n";
    strstrm << w;
    assert(expected == strstrm.str());
    strstrm.str("");




    int rows[] = { 1, 3 };
    v = m.view(rows, rows+2);
    expected =
	"1 0 0 0 0 \n"
	"0 0 0 0 0 \n";
    strstrm << v;
    assert(expected == strstrm.str());
    strstrm.str("");



    //
    // test sortCol:
    //


    Matrix m2(newArrayMatrix(5,5));
    fillMatrix(m2, 0);

    expected = 
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n";
    strstrm << m2;
    assert(expected == strstrm.str());
    strstrm.str("");



    m2(0,2) = 1; 
    m2(1,0) = 1; 
    m2(2,4) = 1;
    m2(3,1) = 1;
    m2(4,3) = 1;

    expected =
	"0 0 1 0 0 \n"
	"1 0 0 0 0 \n"
	"0 0 0 0 1 \n"
	"0 1 0 0 0 \n"
	"0 0 0 1 0 \n";
    strstrm << m2;
    assert(expected == strstrm.str());
    strstrm.str("");


    std::vector<int> pos(5);
    pos[0]= 0;
    pos[1]=20;
    pos[2]=29;
    pos[3]=40;
    pos[4]=45;

    v = m2.view();
    sortColumns(v, pos, 2);
    
    expected =
	"1 0 0 0 0 \n"
	"0 0 0 0 1 \n"
	"0 0 0 1 0 \n"
	"0 1 0 0 0 \n"
	"0 0 1 0 0 \n";

    strstrm << v;
    assert(expected == strstrm.str());
    strstrm.str("");

    Matrix m3(newArrayMatrix(6,5));
    fillMatrix(m3, 0);

    m3(0,2) = 1;
    m3(0,3) = 1;
    m3(1,0) = 1; 
    m3(1,2) = 1;
    m3(1,3) = 1; 
    m3(1,4) = 1;  
    m3(2,3) = 1;
    m3(2,4) = 1;
    m3(1,4) = 1; 
    m3(4,2) = 1;
    m3(4,4) = 1;
    m3(5,2) = 1;

    expected =
	"0 0 1 1 0 \n"  //A
	"1 0 1 1 1 \n"  //A
	"0 0 0 1 1 \n"  //U
	"0 0 0 0 0 \n"  //U
	"0 0 1 0 1 \n"  //U
	"0 0 1 0 0 \n"; //U
    strstrm << m3;
    assert(expected == strstrm.str());
    strstrm.str("");


    MatrixView v3 = m3.view();
    assert(splitOnColumn(v3, 0) == 5);

    expected =
	"0 0 1 1 0 \n"
	"0 0 1 0 0 \n"
	"0 0 0 1 1 \n"
	"0 0 0 0 0 \n"
	"0 0 1 0 1 \n"
	"1 0 1 1 1 \n";
    strstrm << v3;
    assert(expected == strstrm.str());
    strstrm.str("");

    assert(splitOnColumn(v3, 1) == 6);

    assert(splitOnColumn(v3, 2) == 2);

    expected = 
	"0 0 0 0 0 \n"
	"0 0 0 1 1 \n"
	"0 0 1 0 0 \n"
	"0 0 1 1 0 \n"
	"0 0 1 0 1 \n"
	"1 0 1 1 1 \n";

    strstrm << v3;
    assert(expected == strstrm.str());
    strstrm.str("");



    //
    // Test sortRows
    //

    Matrix m4(newArrayMatrix(5,4));
    fillMatrix(m4, 0);
    m4(0, 0) = 1;
    m4(0, 1) = 1;
    m4(0, 3) = 2;

    m4(1, 0) = 1;
    m4(1, 1) = 1;
    m4(1, 3) = 1;

    m4(2, 3) = 2;

    m4(3, 0) = 1;
    m4(3, 3) = 2;

    m4(4, 0) = 1;
    m4(4, 1) = 1;
    m4(4, 3) = 1;

    expected = 
	"1 1 0 2 \n"
	"1 1 0 1 \n"
	"0 0 0 2 \n"
	"1 0 0 2 \n"
    "1 1 0 1 \n";

    MatrixView v4 = m4.view();

    strstrm << v4;
    assert(expected == strstrm.str());
    strstrm.str("");

    sortRows(v4);

    expected = 
    "1 1 0 2 \n"
	"1 1 0 1 \n"
	"1 1 0 1 \n"
	"1 0 0 2 \n"
    "0 0 0 2 \n";

    strstrm << v4;
    assert(expected == strstrm.str());
    strstrm.str("");

    



    /*
    int classes[] = { 1, 1, 0, 0, 0, 0 };
    Classification classification(classes, classes+m3.noRows());

    v = m3.view();
    v.sortCols2(1, pos, classification);

    std::cout << v << std::endl;
    expected =
	"0 1 0 0 1 \n"
	"0 1 1 1 1 \n"
	"0 1 0 1 0 \n"
	"0 0 0 0 0 \n"
	"0 0 0 1 1 \n"
	"0 0 0 0 1 \n";
    strstrm << v;
    assert(expected == strstrm.str());
    strstrm.str("");

    */

    return 0;
}
