/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2007 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * PPH: Copyright (C) 2007 by Zhihong Ding <zhding@ucdavis.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef PPH_HH_INCLUDED
#define PPH_HH_INCLUDED

#include "config.hh"
#include "compatible_cache.hh"

namespace pph {

    /**
     * Class for calculating the compatible regions, using PPH rather
     * than four-gamete testing.
     */
    class CompatibleCache : public ::CompatibleCache {
    public:
	CompatibleCache(const BiRC::SNPFile::ImmutableMatrix &haplotypes);

    private:
	/// calculate the right-vector if it is not in the cache...
	virtual void calc_right(int i) const;
    };

    /**
     * \brief Get the number of classes in the shadow tree for the
     * given region.
     *
     * \param haplotypes	Haplotype matrix for the region.
     * \param left		Left index of the region.
     * \param right		Right index of the region.
     */
    int getNumOfClasses(const BiRC::SNPFile::MatrixView &haplotypes,
			int left, int right);

    /**
     * Store PPH solution number \a solIdx from the shadow tree in \a matrix.
     *
     * \param solIdx		Index of the solution to extract.
     * \param noPPHSolutions	Total number of solutions.
     * \param matrix		Matrix the solution will be written to.
     */
    void getPPHSolution(int solIdx, int noPPHSolutions,
			BiRC::SNPFile::Matrix &matrix);
    
    /**
     * Store ENT phasing solution in \a matrix.
     *
     * \param errorCode		Indicate what global variables (memory) from PPH part to be released.
     * \param matrix		Matrix the solution will be written to.
     */
    void getCCSolution(int errorCode, BiRC::SNPFile::Matrix &matrix);

}

#endif
