/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus,
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "config.hh"
#include "score.hh"
#include "matrix_utils.hh"
#include "tree.hh"
#include "tree_builder.hh"
#include "incompatibility.hh"
#include "case_control_score.hh"
#include "qt_score.hh"

#include <snpfile/array_matrix.hh>
using namespace BiRC::SNPFile;

#include <iostream>

#undef NDEBUG
#include <cassert>

// for linking reasons...
namespace options {
    int unphased = 0;
}

int
main()
{
    Matrix m(newArrayMatrix(10,2));
    m(0,0) = 1; 
    m(1,0) = 1; 
    m(2,0) = 1;
    m(3,0) = 1;
    m(4,0) = 1;
    m(0,1) = 1; 
    m(1,1) = 1; 
    m(2,1) = 1;

    MatrixView v = m.view();
    
    std::vector<int> positions;
    positions.push_back(1);
    positions.push_back(2);
   
    int classes[] = { 0, 0, 0, 1, 1};
    Classification classification(classes, classes+m.noRows());

    Tree *t = 0;

    MatrixView w = m.subMatrix(0, m.noRows(), 0, 1); 
    
    sortColumns(w, positions, 0);
    t = buildTree(w, 0, 0, positions);

#warning This test is disabled.  Enable it again!
#if 0 // FIXME: use this test with the class interface!!!!
    long double BIC1 = BICscore(t, classification, m.noRows());
    long double HQ1  = HQscore(t, classification, m.noRows());
    long double AIC1 = AICscore(t, classification, m.noRows());
    long double Prob1 = probabilityScore(t, classification, m.noRows(), true);
    
    w = m.subMatrix(0, m.noRows(), 0, 2);    
    sortColumns(w, positions, 0);
    t = buildTree(w, 0, 0, positions);
    long double BIC2 = BICscore(t, classification, m.noRows());
    long double HQ2  = HQscore(t, classification, m.noRows());
    long double AIC2 = AICscore(t, classification, m.noRows());
    long double Prob2 = probabilityScore(t, classification, m.noRows(), true);

    assert(BIC1 == BIC2);
    assert(HQ1  == HQ2);
    assert(AIC1 == AIC2);
    assert(Prob1 == Prob2);

    int classes2[] = { 1, 0, 0, 0, 1};
    Classification classification2(classes2, classes2+m.noRows());

    BIC2 = BICscore(t, classification2, m.noRows());
    HQ2  = HQscore(t, classification2, m.noRows());
    AIC2 = AICscore(t, classification2, m.noRows());
    Prob2 = probabilityScore(t, classification2, m.noRows(), true);
    
    assert(BIC1 > BIC2);
    assert(HQ1  > HQ2);
    assert(AIC1 > AIC2);
    assert(Prob1 > Prob2);  
#endif
}
