/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus,
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "tree.hh"

Tree::~Tree() {}

TreeVisitor::~TreeVisitor() {}

InnerNode::~InnerNode() 
{
    delete leftChild;
    delete rightChild;
}

void
InnerNode::acceptVisitor(TreeVisitor &v)
{
    leftChild->acceptVisitor(v);
    rightChild->acceptVisitor(v);
    v.visitInnerNode(*this);
}

inline static void indent(std::ostream &os, int level)
{
    for (int i = 0; i < level; ++i)
	if (i % 2) os << '|';
	else       os << ' ';
}

void
InnerNode::print(std::ostream &os) const
{
    os << '(';
    leftChild->print(os);
    os << ", ";
    rightChild->print(os);
    os << ')';
}



LeafNode::~LeafNode()
{
}

void
LeafNode::acceptVisitor(TreeVisitor &v)
{
    v.visitLeafNode(*this);
}

void
LeafNode::print(std::ostream &os) const
{
    if (i_leafIDs.size() > 1) os << '(';
    const char *sepstr = "";
    for (IDsIterator i = leafIDsBegin(); i != leafIDsEnd(); ++i)
	{
	    os << sepstr << *i; 
	    sepstr = ", ";
	}
    if (i_leafIDs.size() > 1) os << ')';
}
