/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus,
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "config.hh"
#include "tree_builder.hh"
#include <snpfile/matrix.hh>
using namespace BiRC::SNPFile;
#include "matrix_utils.hh"
#include "tree.hh"


Tree *
buildTree(MatrixView &view, int level, int pos, 
	  const std::vector<int>& positions,
	  int minLeafSize)
{
    if (view.noRows() < 1) throw EmptyMatrixView();

    // a single row should just be a singleton cluster
    if (view.noRows() == 1) return new LeafNode(level, view.mappedRow(0));

    int rowSplit = 0, colSplit;
    for (colSplit = 0; colSplit < view.noCols(); ++colSplit)
	{
	    rowSplit = splitOnColumn(view, colSplit);
	    if (0 < rowSplit && rowSplit < view.noRows()) break;
	}

    if ((colSplit == view.noCols())
	// nothing to split on -- cluster them all
	||
	(static_cast<int>(rowSplit) < minLeafSize) || (static_cast<int>(view.noRows()-rowSplit) < minLeafSize))
	// split smaller than desired minimal size
	{
	    std::vector<size_t> leafIDs;
	    for (int i = 0; i < view.noRows(); ++i)
			leafIDs.push_back(view.mappedRow(i));
	    return new LeafNode(level, leafIDs.begin(), leafIDs.end());
	}

    MatrixView left  = view.subMatrix(0,rowSplit,
				      colSplit,view.noCols());
    MatrixView right = view.subMatrix(rowSplit,view.noRows(),
				      colSplit,view.noCols());

    return new InnerNode(level, 
			 buildTree(left, level+1, pos, positions),
			 buildTree(right, level+1, pos,  positions),
			 (positions[view.mappedCol(colSplit)]-positions[pos]));


}
