/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus,
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "config.hh"
#include "tree.hh"
#include "tree_builder.hh"

#include <snpfile/array_matrix.hh>
using namespace BiRC::SNPFile;

#include <iostream>
#include <sstream>

#undef NDEBUG
#include <cassert>

int
main()
{
    
    std::ostringstream strstrm;
    std::string expected;

    Matrix m(newArrayMatrix(7,5));
    m(0,0) = 0; m(0,1) = 1; m(0,2) = 1; m(0,3) = 0; m(0,4) = 0;
    m(1,0) = 1; m(1,1) = 0; m(1,2) = 0; m(1,3) = 0; m(1,4) = 0;
    m(2,0) = 0; m(2,1) = 0; m(2,2) = 0; m(2,3) = 0; m(2,4) = 1;
    m(3,0) = 0; m(3,1) = 0; m(3,2) = 0; m(3,3) = 0; m(3,4) = 1;
    m(4,0) = 0; m(4,1) = 0; m(4,2) = 0; m(4,3) = 1; m(4,4) = 0;
    m(5,0) = 0; m(5,1) = 0; m(5,2) = 0; m(5,3) = 0; m(5,4) = 0;
    m(6,0) = 0; m(6,1) = 0; m(6,2) = 0; m(6,3) = 0; m(6,4) = 0;

    expected =
	"0 1 1 0 0 \n"
	"1 0 0 0 0 \n"
	"0 0 0 0 1 \n"
	"0 0 0 0 1 \n"
	"0 0 0 1 0 \n"
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n";
    strstrm << m;
    assert(expected == strstrm.str());
    strstrm.str("");
    
    MatrixView v = m.view();

    std::vector<int> positions;
    
    for(int i = 0; i < m.noCols(); i++) {
	positions.push_back(i);
    }
    Tree *t = buildTree(v, 0, 0, positions);

    expected = "(((((5, 6), (2, 3)), 4), 0), 1);";
    strstrm << *t;
    assert(expected == strstrm.str());
    strstrm.str("");
    
    delete t;


    Matrix m1(newArrayMatrix(7,5));
    m1(0,0) = 1; m1(0,1) = 0; m1(0,2) = 0; m1(0,3) = 0; m1(0,4) = 0;
    m1(1,0) = 1; m1(1,1) = 0; m1(1,2) = 0; m1(1,3) = 0; m1(1,4) = 0;
    m1(2,0) = 0; m1(2,1) = 0; m1(2,2) = 0; m1(2,3) = 0; m1(2,4) = 1;
    m1(3,0) = 0; m1(3,1) = 0; m1(3,2) = 0; m1(3,3) = 0; m1(3,4) = 1;
    m1(4,0) = 0; m1(4,1) = 0; m1(4,2) = 0; m1(4,3) = 1; m1(4,4) = 0;
    m1(5,0) = 0; m1(5,1) = 0; m1(5,2) = 0; m1(5,3) = 0; m1(5,4) = 0;
    m1(6,0) = 0; m1(6,1) = 0; m1(6,2) = 0; m1(6,3) = 0; m1(6,4) = 0;

    expected =
	"1 0 0 0 0 \n"
	"1 0 0 0 0 \n"
	"0 0 0 0 1 \n"
	"0 0 0 0 1 \n"
	"0 0 0 1 0 \n"
	"0 0 0 0 0 \n"
	"0 0 0 0 0 \n";
    strstrm << m1;
    assert(expected == strstrm.str());
    strstrm.str("");
    
    MatrixView v1 = m1.view();

    Tree *t1 = buildTree(v1, 0, 0, positions);

    expected = "((((6, 5), (2, 3)), 4), (1, 0));";
    strstrm << *t1;
    assert(expected == strstrm.str());
    strstrm.str("");
    
    delete t1;


    Matrix m2(newArrayMatrix(7,5));
    m2(0,0) = 1; m2(0,1) = 0; m2(0,2) = 0; m2(0,3) = 0; m2(0,4) = 0;
    m2(1,0) = 1; m2(1,1) = 0; m2(1,2) = 0; m2(1,3) = 0; m2(1,4) = 0;
    m2(2,0) = 0; m2(2,1) = 0; m2(2,2) = 0; m2(2,3) = 0; m2(2,4) = 1;
    m2(3,0) = 0; m2(3,1) = 0; m2(3,2) = 0; m2(3,3) = 0; m2(3,4) = 1;
    m2(4,0) = 0; m2(4,1) = 0; m2(4,2) = 0; m2(4,3) = 1; m2(4,4) = 0;
    m2(5,0) = 0; m2(5,1) = 0; m2(5,2) = 0; m2(5,3) = 0; m2(5,4) = 0;
    m2(6,0) = 1; m2(6,1) = 0; m2(6,2) = 0; m2(6,3) = 0; m2(6,4) = 0;

    expected =
	"1 0 0 0 0 \n"
	"1 0 0 0 0 \n"
	"0 0 0 0 1 \n"
	"0 0 0 0 1 \n"
	"0 0 0 1 0 \n"
	"0 0 0 0 0 \n"
	"1 0 0 0 0 \n";
    strstrm << m2;
    assert(expected == strstrm.str());
    strstrm.str("");
    
    MatrixView v2 = m2.view();

    Tree *t2 = buildTree(v2, 0, 0, positions);

    expected = "(((5, (3, 2)), 4), (1, 0, 6));";
    strstrm << *t2;
    assert(expected == strstrm.str());
    strstrm.str("");
    
    delete t2;


    try {
	MatrixView w = m.subMatrix(0,0,0,m.noCols());
	buildTree(w, 0, 0, positions);
	assert(!"shouldn't reach this point");
    } catch (EmptyMatrixView&) {}
    
}
