/*
 * Blossoc -- Association Mapping by Incompatibilities
 *
 * Copyright (C) 2006 by Thomas Mailund <mailund@mailund.dk>
 *                    and Søren Besenbacher <besen@birc.au.dk>
 *
 * Copyright (C) 2007 by Bioinformatics Research Center,
 *                       University of Aarhus,
 *                       Contact: Thomas Mailund <mailund@birc.au.dk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "tree.hh"

#include <iostream>
#include <sstream>
#include <cassert>

#undef NDEBUG
#include <cassert>

struct CountVisitor : public TreeVisitor {
    int innerNodes, leafNodes;
    CountVisitor() : innerNodes(0), leafNodes(0) {}
    virtual void visitInnerNode(InnerNode &n) { ++innerNodes; }
    virtual void visitLeafNode(LeafNode &n)   { ++leafNodes; }
};

int
main()
{
    int dummyIds[] = {3, 4, 5};
    Tree *t =
	new InnerNode(0, new InnerNode(1, new LeafNode(2, 0), new LeafNode(2, 1), 0.0),
		      new LeafNode(1, dummyIds,dummyIds+3), 0.0);

    std::ostringstream strstrm;
    std::string expected("((0, 1), (3, 4, 5));");
    //std::cout << *t << " ==?== " << expected << std::endl;
    strstrm << *t;
    assert(strstrm.str() == expected);

    CountVisitor counter;
    t->acceptVisitor(counter);

    assert(counter.innerNodes == 2);
    assert(counter.leafNodes  == 3);

    delete t;
}
